package com.yunzhi.shigongli.service.impl;

import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyResult;
import com.github.binarywang.wxpay.bean.notify.WxPayRefundNotifyResult;
import com.github.binarywang.wxpay.bean.order.WxPayMpOrderResult;
import com.github.binarywang.wxpay.constant.WxPayConstants;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.DateUtils;
import com.yunzhi.shigongli.common.StringUtils;
import com.yunzhi.shigongli.common.WxPayUtils;
import com.yunzhi.shigongli.entity.*;
import com.yunzhi.shigongli.event.PaySuccessEvent;
import com.yunzhi.shigongli.event.RefundSuccessEvent;
import com.yunzhi.shigongli.mapper.*;
import com.yunzhi.shigongli.service.ITaOrderPayService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 订单支付 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Service
public class TaOrderPayServiceImpl extends BaseServiceImpl<TaOrderPayMapper, TaOrderPay> implements ITaOrderPayService {

    @Autowired
    WxPayUtils wxPayUtils;

    @Autowired
    TaOrderPayMapper orderPayMapper;

    @Autowired
    TaOrderMapper taOrderMapper;

    @Autowired
    TaOrderSubMapper taOrderSubMapper;

    @Autowired
    ApplicationEventPublisher eventPublisher;

    @Autowired
    TaSequenceMapper taSequenceMapper;

    @Autowired
    TaShopPackageMapper shopPackageMapper;

    /**
     * 支付回调
     * @param notifyResult
     * @throws Exception
     */
    @Override
    public void notifyOrder(WxPayOrderNotifyResult notifyResult) throws Exception {
        TaOrderPay orderPay = getById(notifyResult.getOutTradeNo());
        String orderId = orderPay.getOrderId();
        String orderSubId = orderPay.getSubOrderId();

        boolean isSuccess = WxPayConstants.ResultCode.SUCCESS.equals(notifyResult.getResultCode());

        orderPay.setPayDate(DateUtils.from(notifyResult.getTimeEnd(), "yyyyMMddHHmmss"));
        orderPay.setStatus(isSuccess ? Constants.PAY_SUCCESS : Constants.PAY_FAIL);
        orderPay.setMessage(isSuccess ? null : notifyResult.getErrCodeDes());

        orderPayMapper.updateById(orderPay);

        // 更新订单为已支付
        if (isSuccess) {
            if (!StringUtils.isEmpty(orderSubId)) {
                taOrderSubMapper.updateStatus(orderSubId, Constants.ORDER_PAYED);
            } else {
                taOrderMapper.updateStatus(orderId, Constants.ORDER_PAYED);
                taOrderSubMapper.updateStatusByOrder(orderId, Constants.ORDER_PAYED);
            }

            PaySuccessEvent evt = new PaySuccessEvent(this, orderPay);
            eventPublisher.publishEvent(evt);
        }
    }

    @Override
    public void refundNotifyOrder(WxPayRefundNotifyResult result) throws Exception {
        WxPayRefundNotifyResult.ReqInfo reqInfo = result.getReqInfo();
        if (null == reqInfo) {
            return;
        }

        // 退款处理中
        if (WxPayConstants.RefundStatus.PROCESSING.equals(reqInfo.getRefundStatus())) {
            return;
        }

        // 退款成功
        boolean isSuccess = WxPayConstants.RefundStatus.SUCCESS.equals(reqInfo.getRefundStatus());

        TaOrderPay orderPay = getById(reqInfo.getOutTradeNo());
        orderPay.setPayDate(DateUtils.from(reqInfo.getSuccessTime(), "yyyyMMddHHmmss"));
        orderPay.setStatus(isSuccess ? Constants.PAY_SUCCESS : Constants.PAY_FAIL);
        orderPay.setMessage(isSuccess ? null : reqInfo.getRefundStatus());

        orderPayMapper.updateById(orderPay);

        String orderId = orderPay.getOrderId();
        String orderSubId = orderPay.getSubOrderId();
        if (isSuccess) {
            if (StringUtils.isEmpty(orderSubId)) {
                taOrderMapper.updateStatus(orderId, Constants.ORDER_REFUND);
                taOrderSubMapper.updateStatusByOrder(orderId, Constants.ORDER_REFUND);
            } else {
                taOrderSubMapper.updateStatus(orderSubId, Constants.ORDER_REFUND);
            }

            RefundSuccessEvent evt = new RefundSuccessEvent(this, orderPay);
            eventPublisher.publishEvent(evt);
        }
    }

    @Override
    public WxPayMpOrderResult prePay(TaPerson person, TaOrder taOrder) throws Exception {
        TaOrderPay orderPay = createByOrder(taOrder);
        orderPayMapper.insert(orderPay);

        return wxPayUtils.createOrder(person, orderPay);
    }

    @Override
    public void refundOrderSub(TaPerson taPerson, TaOrder taOrder, TaOrderSub taOrderSub) throws Exception {
        TaOrderPay originOrderPay = null;

        // 找当前子项目对应的支付记录
        originOrderPay = orderPayMapper.getLastRowByOrderSub(taOrder.getOrderId(), taOrderSub.getSubOrderId());
        if (null != originOrderPay) {
            if (Constants.PAY_SUCCESS.equals(originOrderPay.getStatus())) {
                throw new Exception("未找到当前订单有效的支付记录");
            }
        } else {
            // 找到主订单对应的那个支付记录
            originOrderPay = orderPayMapper.getLastRowByOrder(taOrder.getOrderId());
            if (null != originOrderPay) {
                if (Constants.PAY_SUCCESS.equals(originOrderPay.getStatus())) {
                    throw new Exception("未找到当前订单有效的支付记录");
                }
            } else {
                throw new Exception("未找到当前订单有效的支付记录");
            }
        }

        TaOrderPay orderPay = createByOrderSub(taOrderSub, taPerson);
        // 退款金额 = 订单金额 - 返现金额
        orderPay.setCharges(taOrderSub.getCharges() - taOrderSub.getCashback());

        // 发起退款
        wxPayUtils.refund(originOrderPay, orderPay);
    }

    @Override
    public void refundOrder(TaPerson person, TaOrder taOrder, Integer totalCashback) throws Exception {
        // 找到主订单对应的那个支付记录
        TaOrderPay originOrderPay = orderPayMapper.getLastRowByOrder(taOrder.getOrderId());
        if (null != originOrderPay) {
            if (Constants.PAY_SUCCESS.equals(originOrderPay.getStatus())) {
                throw new Exception("未找到当前订单有效的支付记录");
            }
        } else {
            throw new Exception("未找到当前订单有效的支付记录");
        }

        TaOrderPay orderPay = createByOrder(taOrder);
        // 退款金额 = 订单金额 - 返现金额
        orderPay.setCharges(taOrder.getCharges() - totalCashback);

        // 发起退款
        wxPayUtils.refund(originOrderPay, orderPay);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public WxPayMpOrderResult prePay(TaPerson person, TaOrder taOrder, TaOrderSub taOrderSub) throws Exception {
        TaOrderPay orderPay = createByOrderSub(taOrderSub, person);
        orderPayMapper.insert(orderPay);

        return wxPayUtils.createOrder(person, orderPay);
    }

    private TaOrderPay createByOrder(TaOrder taOrder) throws Exception {
        // 发给微信的订单号为  订单ID-01 这种
        Integer no = taSequenceMapper.nextval(taOrder.getOrderId());
        String payNo = StringUtils.lpad(no.toString(), "0", 2);
        String payId = taOrder.getOrderId() + "-" + payNo;

        TaOrderPay orderPay = new TaOrderPay();
        orderPay.setPayId(payId);
        orderPay.setOrderId(taOrder.getOrderId());
        orderPay.setPersonId(taOrder.getPersonId());
        orderPay.setCharges(taOrder.getCharges());
        orderPay.setDescription(taOrder.getDescription());
        orderPay.setStatus(Constants.STATUS_READY);

        return orderPay;
    }

    private TaOrderPay createByOrderSub(TaOrderSub taOrderSub, TaPerson person) throws Exception {
        // 发给微信的订单号为  订单ID-01 这种
        Integer no = taSequenceMapper.nextval(taOrderSub.getSubOrderId());
        String payNo = StringUtils.lpad(no.toString(), "0", 2);
        String payId = taOrderSub.getSubOrderId() + "-" + payNo;

        TaOrderPay orderPay = new TaOrderPay();
        orderPay.setPayId(payId);
        orderPay.setOrderId(taOrderSub.getOrderId());
        orderPay.setSubOrderId(taOrderSub.getSubOrderId());
        orderPay.setPersonId(person.getPersonId());
        orderPay.setCharges(taOrderSub.getCharges());
        orderPay.setDescription(taOrderSub.getDescription());
        orderPay.setStatus(Constants.STATUS_READY);

        return orderPay;
    }
}
